__author__ = "Christian Kongsgaard"
__license__ = 'GNU GPLv3'

# -------------------------------------------------------------------------------------------------------------------- #
# Imports

# Module imports
import numpy as np
import multiprocessing
from scipy.optimize import brentq


# Livestock imports


# -------------------------------------------------------------------------------------------------------------------- #
# Livestock Air Functions

def new_temperature_and_relative_humidity(folder: str) -> bool:
    """
    Calculates a new temperatures and relative humidity for air volumes.

    :param folder: Path to folder containing case files.
    :type folder: str
    :return: True
    :rtype: bool
    """

    # Helper functions
    def get_files(folder_: str) -> tuple:
        """
        Reads the case files and returns them as a tuple of lists
        :param folder_: Path to case folder
        :return: Tuple of lists
        """

        # Load
        air_temperature_ = np.loadtxt(folder_ + '/temperature.txt', delimiter=',')
        air_relative_humidity_ = np.loadtxt(folder_ + '/relative_humidity.txt', delimiter=',')
        wind_speed_ = np.loadtxt(folder_ + '/wind_speed.txt', delimiter=',')
        area_ = np.loadtxt(folder_ + '/area.txt', delimiter=',')
        height_top_, height_stratification_ = np.loadtxt(folder_ + '/heights.txt', delimiter=',')
        vapour_flux_ = np.loadtxt(folder_ + '/vapour_flux.txt', delimiter=',')
        cpu_ = np.loadtxt(folder + '/cpu.txt', delimiter=',')

        return (air_temperature_, air_relative_humidity_, wind_speed_, area_, height_top_, height_stratification_,
                vapour_flux_, int(cpu_))

    def reconstruct_results(folder_, processed_rows_):
        """
        Collects and saves the results when they have been simulated.

        :param folder_: Folder where the results should be saved
        :param processed_rows_: Processed result rows.
        :return: True
        """

        # Sort row list
        sorted_rows = sorted(processed_rows_)

        # create lists
        temperature_ = []
        relative_humidity_ = []
        latent_heat_flux_ = []
        vapour_flux_ = []

        for row in sorted_rows:
            temperature_.append(row[1])
            relative_humidity_.append(row[2])
            latent_heat_flux_.append(row[3])
            vapour_flux_.append(row[4])

        np.savetxt(folder_ + '/temperature_results.txt', temperature_, delimiter=',', fmt='%.4f')
        np.savetxt(folder_ + '/relative_humidity_results.txt', relative_humidity_, delimiter=',', fmt='%.4f')
        np.savetxt(folder_ + '/latent_heat_flux_results.txt', latent_heat_flux_, delimiter=',', fmt='%.4f')
        np.savetxt(folder_ + '/vapour_flux_results.txt', vapour_flux_, delimiter=',', fmt='%.4f')

        return True

    # Run function
    temperature, relative_humidity, wind_speed, area, height_top, height_stratification, vapour_flux, cpu = \
        get_files(folder)

    rows_ = [i
             for i in range(0, len(vapour_flux) - 1)]

    input_packages = [(index,
                       temperature[index],
                       convert_relative_humidity_to_unitless(relative_humidity[index]),
                       vapour_flux[index],
                       wind_speed[index],
                       area,
                       height_stratification,
                       height_top)
                      for index in rows_]

    pool = multiprocessing.Pool(processes=cpu)
    processed_rows = pool.map(run_row, input_packages)
    reconstruct_results(folder, processed_rows)

    return True


def max_possible_vapour_flux(vapour_mass_flux: float, volume: float,
                             temperature_in_kelvin: float, vapour_pressure: float) -> float:
    """
    Computes the difference between the saturated vapour pressure of an air volume after adding the vapour
    and latent heat flux to an air volume and the actual vapour pressure of an air volume.

    :param vapour_mass_flux: Vapour mass flux in kg/h
    :type vapour_mass_flux: float
    :param volume: Air volume in m\ :sup:`3`\
    :type volume: float
    :param temperature_in_kelvin: Current temperature in K
    :type temperature_in_kelvin: float
    :param vapour_pressure: Current vapour pressure in Pa
    :type vapour_pressure: float
    :return: Difference between the saturated vapour pressure after adding the vapour and latent heat flux to the air
    volume and the actual vapour pressure of the air volume.
    :rtype: float
    """

    # Solve Function
    new_temperature = new_mean_temperature(volume,
                                           temperature_in_kelvin,
                                           latent_heat_flux(vapour_mass_flux))

    saturated_pressure = saturated_vapour_pressure(new_temperature)
    actual_pressure = new_mean_vapour_pressure(volume, new_temperature, vapour_pressure, vapour_mass_flux)
    return saturated_pressure - actual_pressure


def compute_temperature_relative_humidity(temperature_in_k: np.array, relative_humidity: np.array,
                                          vapour_mass_flux: np.array, volume: np.array) -> tuple:
    """
    Computes the coupled relative humidity and temperature of an air volume given a vapour flux. The vapour pressure is
    capped off so it can not exceed the saturated vapour pressure. This potentially means that not the whole amount of
    vapour flux will be used.

    :param temperature_in_k: Current temperature of the air volume in K
    :type temperature_in_k: numpy.array
    :param relative_humidity: Current relative humidity of the air volume as unit less.
    :type relative_humidity: numpy.array
    :param vapour_mass_flux: Vapour mass flux to be added to the air volume in kg/h.
    :type vapour_mass_flux: numpy.array
    :param volume: Air volume in m\ :sup:`3`\
    :type volume: numpy.array
    :return: Tuple containing the new temperature in K, new relative humidity as unit less, the latent heat used and
     the vapour flux used.
    :rtype: tuple
    """

    # Check if all cells have a vapour pressure below saturation after alteration of temp & relhum
    new_temperature_check = new_mean_temperature(volume, temperature_in_k,
                                                 latent_heat_flux(vapour_mass_flux))

    vapour_pressure = relative_humidity_to_vapour_pressure(relative_humidity,
                                                           temperature_in_k)

    available_vapour_mass_flux = []
    for i in range(0, len(vapour_mass_flux)):
        new_vapour_pressure = new_mean_vapour_pressure(volume[i],
                                                       new_temperature_check[i],
                                                       vapour_pressure,
                                                       vapour_mass_flux[i])

        new_saturated_pressure = saturated_vapour_pressure(new_temperature_check[i])

        if new_vapour_pressure <= new_saturated_pressure:
            available_vapour_mass_flux.append(vapour_mass_flux[i])
        else:
            possible_vapour_mass_flux = brentq(max_possible_vapour_flux,
                                               -1,
                                               vapour_mass_flux[i],
                                               args=(volume[i], temperature_in_k, vapour_pressure))

            if possible_vapour_mass_flux < vapour_mass_flux[i]:
                available_vapour_mass_flux.append(possible_vapour_mass_flux)
            else:
                # Should be a useless statement?
                available_vapour_mass_flux.append(vapour_mass_flux[i])

    available_vapour_mass_flux = np.array(available_vapour_mass_flux).flatten()
    latent_heat = latent_heat_flux(available_vapour_mass_flux)
    new_temperature_ = new_mean_temperature(volume, temperature_in_k, latent_heat)
    new_relative_humidity = new_mean_relative_humidity(volume,
                                                       new_temperature_,
                                                       vapour_pressure,
                                                       available_vapour_mass_flux)
    return (new_temperature_,
            new_relative_humidity,
            latent_heat,
            available_vapour_mass_flux)


def run_row(input_package: list) -> tuple:
    """
    Calculates a new temperatures and relative humidity for a row. A row represent all cells to a given time.

    :param input_package: Input package with need inputs.
    :type input_package: list
    :return: The row on which the calculation was performed.
    :rtype: tuple
    """

    # unpack
    (row_index, temperature_time, relative_humidity_time, vapour_flux_time_row, wind_speed, area,
     height_stratification, height_top) = input_package

    # new mean temperature i K
    air_temperature_in_k = celsius_to_kelvin(temperature_time)
    flux_corrected_volume = area * height_top + wind_speed_to_flux(wind_speed, height_top, diameter_from_area(area))

    # Get temperature and relative_humidity
    [temperature_row,
     relative_humidity_row,
     heat_flux_time_row,
     used_vapour_flux] = compute_temperature_relative_humidity(air_temperature_in_k,
                                                               relative_humidity_time,
                                                               convert_vapour_flux_to_kgh(vapour_flux_time_row),
                                                               flux_corrected_volume)

    # new stratified relative humidity
    stratified_relative_humidity_row = stratification(height_stratification,
                                                      relative_humidity_row,
                                                      height_top,
                                                      relative_humidity_time
                                                      )

    # new stratified temperature in C
    stratified_temperature_row = stratification(height_stratification,
                                                kelvin_to_celsius(temperature_row),
                                                height_top,
                                                temperature_time)

    return (row_index,
            stratified_temperature_row,
            convert_relative_humidity_to_percentage(stratified_relative_humidity_row),
            heat_flux_time_row,
            used_vapour_flux)


def latent_heat_flux(vapour_mass_flux: np.array) -> np.array:
    """
    Computes the latent heat flux related to a certain evapotranspiration flux.
    The latent heat flux is negative if the vapour flux is positive.

    Source: Manickathan, L. et al., 2018.
    Parametric study of the influence of environmental factors and tree properties on the
    transpirative cooling effect of trees. Agricultural and Forest Meteorology.

    :param vapour_mass_flux: Vapour volume flux in kg/h
    :type vapour_mass_flux: numpy.array
    :return: Latent heat flux in J/h.
    :rtype: numpy.array
    """

    latent_heat_of_vaporization = 2.5 * 10 ** 6  # J/kg

    return np.negative(latent_heat_of_vaporization * vapour_mass_flux)  # J/h


def convert_vapour_flux_to_kgh(vapour_flux: np.array) -> np.array:
    """
    Converts a vapour flux from m\ :sup:`3`\/day to kg/h
    Density of water: 1000kg/m\ :sup:`3`\
    Hours per day: 24h/day
    Conversion: 1000kg/m\ :sup:`3`\ / 24h/day

    :param vapour_flux: Vapour flux in m\ :sup:`3`\/day
    :type vapour_flux: numpy.array
    :return: Vapour flux in kg/h
    :rtype: numpy.array
    """

    return vapour_flux * 1000 / 24


def convert_relative_humidity_to_unitless(rh: np.array) -> np.array:
    """
    Converts relative humidity from percentage to an unit less number.

    :param rh: Relative humidity in %
    :type rh: numpy.array
    :return: Relative humidity as unit less
    :rtype: numpy.array
    """

    return rh / 100


def convert_relative_humidity_to_percentage(rh: np.array) -> np.array:
    """
    Converts relative humidity from percentage to an unit less number.

    :param rh: Relative humidity as unitless
    :type rh: numpy.array
    :return: Relative humidity in percentage
    :rtype: numpy.array
    """

    return rh * 100


def new_mean_relative_humidity(volume: np.array, temperature_internal: np.array,
                               vapour_pressure_external: np.array, vapour_production: np.array) -> np.array:
    """
    Computes a new mean vapour pressure and converts it in to a relative humidity.

    Source: Peuhkuri, Ruut, and Carsten Rode. 2016.
    “Heat and Mass Transfer in Buildings.”

    :param volume: Air volume in m\ :sup:`3`
    :type volume: numpy.array
    :param temperature_internal: External temperature in K
    :type temperature_internal: numpy.array
    :param vapour_pressure_external: External vapour pressure in Pa
    :type vapour_pressure_external: numpy.array
    :param vapour_production: Vapour production in kg/h
    :type vapour_production: numpy.array
    :return: Relative humidity - unit less
    :rtype: numpy.array
    """

    vapour_pressure = new_mean_vapour_pressure(volume, temperature_internal, vapour_pressure_external,
                                               vapour_production)

    return vapour_pressure_to_relative_humidity(vapour_pressure, temperature_internal)


def new_mean_vapour_pressure(volume: np.array, temperature: np.array,
                             vapour_pressure_external: np.array, vapour_production: np.array) -> np.array:
    """
    Calculates a new vapour pressure for the volume.

    Source: Peuhkuri, Ruut, and Carsten Rode. 2016.
    “Heat and Mass Transfer in Buildings.”

    :param volume: Volume in m\ :sup:`3`
    :type volume: numpy.array
    :param temperature: Temperature in K
    :type temperature: numpy.array
    :param vapour_pressure_external: External vapour pressure in Pa
    :type vapour_pressure_external: numpy.array
    :param vapour_production: Vapour production in kg/h
    :type vapour_production: numpy.array
    :return: New vapour pressure in Pa
    :rtype: numpy.array
    """

    gas_constant_vapour = 461.5  # J/kgK
    density_vapour = vapour_pressure_external / (gas_constant_vapour * temperature)  # kg/m3
    mass_vapour = density_vapour * volume  # kg
    new_mass_vapour = mass_vapour + vapour_production  # kg
    vapour_pressure = (new_mass_vapour * gas_constant_vapour * temperature) / volume  # Pa

    return vapour_pressure  # Pa


def new_mean_temperature(volume: np.array, temperature: np.array, heat: np.array) -> np.array:
    """
    Calculates a new mean temperature for the volume.

    Source: Peuhkuri, Ruut, and Carsten Rode. 2016.
    “Heat and Mass Transfer in Buildings.”

    :param volume: Volume in m\ :sup:`3`
    :type volume: numpy.array
    :param temperature: Temperature at the top of the air volume in K
    :type temperature: numpy.array
    :param heat: Added heat to the air volume in J/h
    :type heat: numpy.array
    :return: Temperature in K
    :rtype: numpy.array
    """

    specific_heat_capacity = 1005  # J/kgK
    density_air = 1.29 * 273.15 / temperature  # kg/m^3
    energy_air = volume * specific_heat_capacity * density_air * temperature  # J

    # Create a check to make sure that the temperature can not go below 0C
    new_temperature = np.maximum((energy_air + heat) / (volume * density_air * specific_heat_capacity), 273.15)

    return new_temperature


def celsius_to_kelvin(celsius: float) -> float:
    """
    Converts a temperature in Celsius to Kelvin. `Source`__

    __ https://en.wikipedia.org/wiki/Celsius

    :param celsius: Temperature in Celsius
    :type celsius: float
    :return: Temperature in Kelvin
    :rtype: float
    """

    kelvin = celsius + 273.15

    return kelvin


def kelvin_to_celsius(kelvin: float) -> float:
    """
    Converts a temperature in Kelvin to Celsius. `Source`__

    __ https://en.wikipedia.org/wiki/Celsius

    :param kelvin: Temperature in Kelvin
    :type kelvin: float
    :return: Temperature in Celsius
    :rtype: float
    """

    celsius = kelvin - 273.15
    return celsius


def vapour_pressure_to_relative_humidity(vapour_pressure: float, temperature: float) -> float:
    """
    Convert vapour pressure to relative humidity given a air temperature

    Source: Peuhkuri, Ruut, and Carsten Rode. 2016.
    “Heat and Mass Transfer in Buildings.”

    :param vapour_pressure: Vapour pressure in Pa
    :type vapour_pressure: float
    :param temperature: Air temperature in K
    :type temperature: float
    :return: Relative humidity as unitless
    :rtype: float
    """

    return vapour_pressure / saturated_vapour_pressure(temperature)  # -


def relative_humidity_to_vapour_pressure(relative_humidity: float, temperature: float) -> float:
    """
    Convert relative humidity to vapour pressure given a air temperature.

    Source: Peuhkuri, Ruut, and Carsten Rode. 2016.
    “Heat and Mass Transfer in Buildings.”

    :type temperature: float
    :type relative_humidity: float
    :param relative_humidity: Relative humidity - unitless
    :param temperature: Air temperature in K
    :return: Vapour pressure in Pa
    :rtype: float
    """

    return relative_humidity * saturated_vapour_pressure(temperature)  # Pa


def stratification(height: float, value_mean: float, height_top: float, value_top: float) -> float:
    """
    Calculates the stratification of the temperature or relative humidity of the air volume.

    :param height: Height at which the stratification value is wanted in m.
    :type height: float
    :param value_mean: Mean value of the air volume. Assumed equal to the value at half of the height of the air volume.
    :type value_mean: float
    :param height_top: Height at the top of the boundary in m.
    :type height_top: float
    :param value_top: Value at the top of the air volume
    :type value_top: float
    :return: Value at desired height.
    :rtype: float
    """

    return value_mean - 2 * height * (value_mean - value_top) / height_top


def saturated_vapour_pressure(temperature: float) -> float:
    """
    Computes the saturated vapour pressure for a given temperature.
    Source: Peuhkuri, Ruut, and Carsten Rode. 2016.
    “Heat and Mass Transfer in Buildings.”

    :param temperature: Temperature in Kelvin
    :type temperature: float
    :return: Vapour pressure in Pa
    :rtype: float
    """

    # Make check. -109.8C is the temperature where the air can hold no water.
    temperature_in_celsius = np.maximum(kelvin_to_celsius(temperature), -109)

    return 288.68 * (1.098 + temperature_in_celsius / 100) ** 8.02


def wind_speed_to_hour_flux(wind_speed: float) -> float:
    """
    | Converts wind speed into a hourly flux.
    | m/s to m\ :sup:`3` /h =>
    | m/s to m\ :sup:`3` /s = 1\ :sup:`2`
    | m\ :sup:`3`/s to m\ :sup:`3`/h = 3600s/h

    :param wind_speed: Wind speed in m/s
    :type wind_speed: float
    :return: Wind flux in m\ :sup:`3`/h
    :rtype: float
    """

    return wind_speed * 3600


def diameter_from_area(area: np.array) -> np.array:
    """
    | Computes the diameter from a given area of a circle.
    | A = :math:`{\\pi}` * (d/2)\ :sup:`2` =>
    | d = :math:`{\\sqrt{4*A/{\\pi}}}`

    :param area: Area of a circle in m
    :type area: numpy.array
    :return: Diameter in m
    :rtype: numpy.array
    """

    return np.sqrt(4 * area / np.pi)


def wind_speed_to_flux(wind_speed: np.array, height: np.array, cross_section: np.array) -> np.array:
    """
    Converts a wind speed through an area to a wind flux.

    :param wind_speed: Wind speed in m/s
    :type wind_speed: numpy.array
    :param height: Height of the area in m
    :type height: numpy.array
    :param cross_section: Width of the area in m
    :type cross_section: numpy.array
    :return: Wind flux in m\ :sup:`3`/h
    :rtype: numpy.array
    """

    return wind_speed_to_hour_flux(wind_speed) * height * cross_section



__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# -------------------------------------------------------------------------------------------------------------------- #
# Imports


# Module imports
import pytest
import platform
import os

# Livestock imports
from livestock import flow

# -------------------------------------------------------------------------------------------------------------------- #
# CMF Functions and Classes


@pytest.mark.skipif(platform.system() == 'Linux',
                    reason='Test should only run locally')
def test_flow_from_centers(drain_mesh):

    flow.flow_from_centers(drain_mesh)

    assert os.path.exists(os.path.join(drain_mesh, 'results.json'))



__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# ---------------------------------------------------------------------------- #
# Imports

# Module imports
import pytest
import os
import shutil
import cmf

# Livestock imports
from livestock import geometry
from livestock import hydrology
from livestock import log


# ---------------------------------------------------------------------------- #
# CMF Functions and Classes


@pytest.fixture()
def data_folder():
    parent = os.path.dirname(__file__)
    return os.path.join(parent, 'test_data')


@pytest.fixture(params=[0, 1])
def obj_file_paths(data_folder, request):
    obj_folder = os.path.join(data_folder, 'obj_to_shp')
    return os.path.join(obj_folder, f'mesh_{request.param}.obj')


@pytest.fixture()
def shapely_polygons(obj_file_paths):
    return geometry.obj_to_polygons(obj_file_paths)


@pytest.fixture(params=['run_off_0', 'run_off_1'])
def input_files(tmpdir, data_folder, request):
    test_folder = tmpdir.mkdir('test')
    data_path = os.path.join(data_folder, request.param)
    files = os.listdir(data_path)

    for file in files:
        shutil.copyfile(os.path.join(data_path, file),
                        os.path.join(test_folder, file))

    return test_folder


@pytest.fixture()
def cmf_data(input_files):
    return hydrology.load_cmf_files(input_files)


@pytest.fixture()
def project_with_cells(cmf_data):
    (ground_list, mesh_paths, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data

    project = cmf.project()

    return hydrology.mesh_to_cells(project, mesh_paths, False)


@pytest.fixture()
def retention_curve(cmf_data):
    (ground_list, mesh_path, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data

    curve_dict = ground_list[0]['ground_type']['retention_curve']
    return hydrology.create_retention_curve(curve_dict)


@pytest.fixture()
def solve_ready_project(cmf_data):
    (ground_list, mesh_path, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data

    project = cmf.project()
    project, mesh_info = hydrology.mesh_to_cells(project, mesh_path, False)

    for ground in ground_list:
        hydrology.configure_cells(project, ground, mesh_info[ground['mesh']])

    if trees_dict:
        for key in trees_dict.keys():
            hydrology.add_tree_to_project(project,
                                          trees_dict[key]['face_index'],
                                          trees_dict[key]['property'])

    # Create the weather
    if weather_dict:
        hydrology.create_weather(project)

    # Create boundary conditions
    if boundary_dict:
        hydrology.create_boundary_conditions(project)

    return project, solver_settings, outputs


@pytest.fixture()
def drain_mesh(data_folder):

    yield os.path.join(data_folder, 'drainage_flow')

    os.remove(os.path.join(data_folder, 'results.json'))


@pytest.fixture()
def mock_solver(monkeypatch):

    class MockClass:
        def __init__(self, project, tolerance):
            self.project = project
            self.tol = tolerance
            self.t = None

        def run(self, start_time, end_time, time_step):
            current_time = start_time
            while current_time < end_time:
                yield current_time
                current_time += time_step

    monkeypatch.setattr(cmf, 'CVodeIntegrator', MockClass)


@pytest.fixture()
def mock_gather_results(monkeypatch):

    def mock_return(cmf_project, results, time):
        pass

    monkeypatch.setattr(hydrology, 'gather_results', mock_return)


@pytest.fixture(autouse=True)
def disable_logger(monkeypatch):

    def mock_return():

        class MockLog:

            def debug(self):
                return None

            def info(self):
                return None

            def warning(self):
                return None

            def error(self):
                return None

        return MockLog()

    monkeypatch.setattr(log, 'livestock_logger', mock_return)



import cmf
import numpy as np
from matplotlib.animation import FuncAnimation
import datetime
# Import the hill_plot
from cmf.draw import hill_plot
# Import some matplotlib stuff
from matplotlib.pylab import figure,show,cm


def z(x):
    return 10/(1+np.exp((x-100)/30))
# create a project
p = cmf.project()

for i in range(20):
    x = i * 10.
    # create a cell with surface storage
    c = p.NewCell(x,0,z(x),100,True)

for c_upper, c_lower in zip(p[:-1], p[1:]):
    c_upper.topology.AddNeighbor(c_lower, 10.)

# Customize cells
for c in p:
    # create layers
    for d in [0.02,0.05,0.1,0.2,0.3,0.5,0.75,1.0,1.25,1.5,1.75,2.]:
        rc = cmf.VanGenuchtenMualem(Ksat=50*np.exp(-d),alpha=0.1,n=2.0,phi=0.5)
        rc.w0 = 0.9996
        c.add_layer(d,rc)
    # set initial conditions
    c.saturated_depth=2.
    # use Richards connection
    c.install_connection(cmf.Richards)
    c.install_connection(cmf.GreenAmptInfiltration)
    # Add more connections here... (eg. ET, snowmelt, canopy overflow)

cmf.connect_cells_with_flux(p,cmf.Darcy)
cmf.connect_cells_with_flux(p,cmf.KinematicSurfaceRunoff)

for c in p:
    c.set_rainfall(10.)

outlet = p.NewOutlet('outlet',200,0,0)
for l in p[-1].layers:
    # create a Darcy connection with 10m flow width between each soil layer
    # and the outlet
    cmf.Darcy(l,outlet,FlowWidth=10.)
cmf.KinematicSurfaceRunoff(p[-1].surfacewater,outlet,10.)

solver = cmf.CVodeIntegrator(p,1e-9)
solver.t = datetime.datetime(2012,1,1)



# Create a new matplotlib figure fig
fig = figure(figsize=(16,9))
# Create a subplot with a light grey background
ax = fig.add_subplot(111,axisbg='0.8')
# Create the hillplot for water filled pore space with using a yellow to green to blue colormap
image = hill_plot(p,solver.t)
# Set the scale for the arrows. This value will change from model to model.
image.scale = 100.
# white arrows are nicer to see
image.q_sub.set_facecolor('w')

def run(frame):
    # Run model for one day
    t = solver(cmf.day)
    # Update image
    image(t)


animation = FuncAnimation(fig,run,repeat=False,
                          frames=365)
show()


import progressbar
import time
import datetime


def format_custom_text():
    format_custom_text = progressbar.FormatCustomText(
        'Spam: %(spam).1f kg, eggs: %(eggs)d',
        dict(
            spam=0.25,
            eggs=3,
        ),
    )

    bar = progressbar.ProgressBar(widgets=[
        format_custom_text,
        ' :: ',
        progressbar.Percentage(),
    ])
    for i in bar(range(25)):
        format_custom_text.update_mapping(eggs=i * 2)
        time.sleep(0.1)


def format_label():
    widgets = [progressbar.FormatLabel(
        'Processed: %(value)d lines (in: %(elapsed)s)')]
    bar = progressbar.ProgressBar(widgets=widgets)
    for i in bar((i for i in range(15))):
        time.sleep(0.1)


def print_output():
    for i in progressbar.progressbar(range(100), redirect_stdout=True):
        print('Some text', i)
        time.sleep(0.1)

def cmf_bar():

    widgets=[' [', progressbar.Timer(), '] ', progressbar.Bar(), ' [', progressbar.AdaptiveETA(), ' ]']
    bar = progressbar.ProgressBar(max_value=20, widgets=widgets)
    for i in range(20):
        time.sleep(0.1)
        bar.update()


#format_custom_text()
cmf_bar()
#format_label()
#print_output()


__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# -------------------------------------------------------------------------------------------------------------------- #
# Imports

# Module imports
import os
import shapefile

# Livestock imports
from livestock import geometry


# -------------------------------------------------------------------------------------------------------------------- #
# CMF Functions and Classes


def test_shapely_to_pyshp(shapely_polygons):

    pyshapes = []
    for polygon in shapely_polygons:
        pyshapes.append(geometry.shapely_to_pyshp(polygon))

    assert pyshapes
    for shape in pyshapes:
        assert isinstance(shape, shapefile._Shape)
        assert isinstance(shape.parts, list)
        assert shape.parts[0] == 0
        assert isinstance(shape.points, tuple)
        assert shape.shapeType == 15


def test_obj_to_polygons(obj_file_paths):
    polygon_list = geometry.obj_to_polygons(obj_file_paths)

    assert polygon_list
    assert isinstance(polygon_list, list)

    for polygon in polygon_list:
        assert polygon.is_valid
        assert polygon.type == 'Polygon'
        assert polygon.has_z


def test_centroid_z(shapely_polygons):

    for polygon in shapely_polygons:
        centroid = geometry.centroid_z(polygon)
        assert centroid
        assert isinstance(centroid, float)


def test_obj_to_shp(obj_file_paths, tmpdir):
    test_folder = tmpdir.mkdir('test')
    file = os.path.join(test_folder, 'mesh_shape.shp')
    geometry.obj_to_shp(obj_file_paths, file)

    assert os.path.isfile(file)
    assert os.stat(file).st_size



import multiprocessing
import numpy as np

def doubler(number):
    return number[0] * 2


numbers = [[5,0], [10,0], [20,]]


if __name__ == '__main__':

    pool = multiprocessing.Pool(processes=3)
    result = pool.map(doubler, numbers)
    print(result)


import bpy
import bmesh
import json

def clean():
    for o in bpy.data.objects:
        if o.type == 'MESH':
            o.select = True
        else:
            o.select = False

    # call the operator once
    bpy.ops.object.delete()


clean()

file_obj = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\tests\test_data\drainage_flow\drain_mesh.obj'

bpy.ops.import_scene.obj(filepath=file_obj, axis_forward='X', axis_up='Z', )
imported_mesh = bpy.context.selected_objects[-1]

me = imported_mesh.data
bm = bmesh.new()

# Get a BMesh representation
bm.from_mesh(me)

lowest_neighbour = []
for face in bm.faces:
    linked_faces = set(f
                       for v in face.verts
                       for f in v.link_faces)
    centers = [[linked_face.index, tuple(linked_face.calc_center_median())]
               for linked_face in linked_faces]

    sorted_centers = sorted(centers, key=lambda v: v[1][2])

    if face.calc_center_median().z <= sorted_centers[0][1][2]:
        lowest_neighbour.append(None)
    else:
        lowest_neighbour.append(sorted_centers[0])

#print(lowest_neighbour)


def get_curve_points(start_index, point_list):

    curve_points = []
    next_index = start_index
    while True:
        pt = point_list[next_index]

        if pt:
            next_index = pt[0]
            curve_points.append(pt[1])
        else:
            return curve_points


curves = []
for face_index in range(len(bm.faces)):
    curves.append(get_curve_points(face_index, lowest_neighbour))

outfile = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\tests\test_data\drainage_flow\result.json'

with open(outfile, 'w') as file:
    json.dump(curves, file)



import air as air
import shutil



folder_path = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\livestock\test\test_data\new_air_conditions'

if __name__ == '__main__':
    air.new_temperature_and_relative_humidity(folder_path)



# Imports
from pathlib import Path
home_user = str(Path.home())
from livestock.hydrology import CMFModel
# Run CMF Model
folder = home_user + '/livestock/ssh'
model = CMFModel(folder)
model.run_model()
# Announce that template finished and create out file
print('Finished with template')
file_obj = open('out.txt', 'w')
file_obj.close()


__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# -------------------------------------------------------------------------------------------------------------------- #
# Imports


# Module imports
try:
    import bpy
    import bmesh
except ModuleNotFoundError:
    pass

import json
import os
import typing

# Livestock imports
from livestock import blender

# -------------------------------------------------------------------------------------------------------------------- #
# Flow Functions and Classes


def get_curve_points(start_point, start_index: int, point_list) \
        -> typing.List[tuple]:

    curve_points = [start_point, ]
    next_index = start_index

    while True:
        pt = point_list[next_index]

        if pt:
            next_index = pt[0]
            curve_points.append(pt[1])
        else:
            return curve_points


def flow_from_centers(folder: str) -> None:
    """Blender depend"""

    blender.clean()

    mesh_file = os.path.join(folder, 'drain_mesh.obj')
    result_file = os.path.join(folder, 'results.json')
    bpy.ops.import_scene.obj(filepath=mesh_file, axis_forward='X', axis_up='Z')
    imported_mesh = bpy.context.selected_objects[-1]

    # Get a BMesh representation
    me = imported_mesh.data
    bm = bmesh.new()
    bm.from_mesh(me)
    bm.faces.ensure_lookup_table()

    lowest_neighbour = []
    for face in bm.faces:
        linked_faces = set(f
                           for v in face.verts
                           for f in v.link_faces)
        centers = [[linked_face.index, tuple(linked_face.calc_center_median())]
                   for linked_face in linked_faces]

        sorted_centers = sorted(centers, key=lambda v: v[1][2])

        if face.calc_center_median().z <= sorted_centers[0][1][2]:
            lowest_neighbour.append(None)
        else:
            lowest_neighbour.append(sorted_centers[0])

    curves = []
    for face_index in range(len(bm.faces)):
        start_point = tuple(bm.faces[face_index].calc_center_median())
        curves.append(get_curve_points(start_point, face_index,
                                       lowest_neighbour))

    with open(result_file, 'w') as outfile:
        json.dump(curves, outfile)



import win_cmf as wc

wc.cell_results('surface_water_flux',
                r'C:\Users\Christian\Desktop\Livestock\CMF_Slope\results.xml',
                r'C:\Users\Christian\Desktop\Livestock\CMF_Slope')


# Imports
import livestock.air as la
# Run function
if __name__ == '__main__':
	la.new_temperature_and_relative_humidity(r'C:\Users\Christian\Dropbox\Uddannelse\DTU\Projekter\Speciale\Wardi\Wardi_No_Water_v3/NewAir')
# Announce that template finished and create out file
print('Finished with template')


import shapely
import numpy as np
import livestock.geometry as geo

path = r'C:\Users\Christian\Desktop\test_mesh.obj'

shape = geo.obj_to_polygons(path)
point = list(shape[0].exterior.coords)


def centroid_z(polygon):
    z_values = []
    for pt in polygon.exterior.coords:
        z_values.append(pt[2])

    mean_z = sum(z_values)/len(z_values)

    return mean_z



import cmf
from numpy import transpose
from matplotlib import pyplot as plt

p = cmf.project()
# Create cell with 1000m2 and surface water storage
c = p.NewCell(0,0,1,1000,True)
# Set puddle depth to 2mm
c.surfacewater.puddledepth = 0.002
# Add a thick layer, low conductivity. Use Green-Ampt-Infiltration
c.add_layer(0.1, cmf.VanGenuchtenMualem(Ksat=0.1))
c.install_connection(cmf.GreenAmptInfiltration)
# Create outlet, 10 m from cell center, 1m below cell
outlet = p.NewOutlet('outlet',10,0,0)
# Create connection, distance is calculated from position
con = cmf.KinematicSurfaceRunoff(c.surfacewater,outlet,flowwidth=10)
# set rainfall, a good shower to get surface runoff for sure (100mm/day)
c.set_rainfall(100.)
# Create a solver
solver = cmf.CVodeIntegrator(p,1e-8)

# Calculate results
Vsoil, Vsurf, qsurf,qinf = transpose([(c.layers[0].volume, c.surfacewater.volume, outlet(t), c.layers[0](t))
                             for t in solver.run(cmf.Time(1,1,2012),cmf.Time(2,1,2012),cmf.min)])
# Present results
ax1=plt.subplot(211)
plt.plot(Vsurf,label='Surface')
plt.plot(Vsoil,label='Soil')
plt.ylabel('Water content in mm')
plt.legend(loc=0)
plt.subplot(212,sharex=ax1)
plt.plot(qsurf,label='Surface')
plt.plot(qinf,label='Soil')
plt.ylabel('Flux in mm/day')
plt.xlabel('Time in minutes')
plt.legend(loc=0)
plt.show()


import cmf
from numpy import transpose
from matplotlib import pyplot as plt
import datetime

p = cmf.project()
# Create cell with 1000m2 and surface water storage
c = p.NewCell(0,0,1,1000,True)
# Set puddle depth to 2mm
c.surfacewater.puddledepth = 0.002
# Add a thick layer, low conductivity. Use Green-Ampt-Infiltration
c.add_layer(0.1, cmf.VanGenuchtenMualem(Ksat=0.1))
c.install_connection(cmf.GreenAmptInfiltration)

# Create a Neumann Boundary condition connected to W1
In = cmf.NeumannBoundary.create(c.surfacewater)
# Create a timeseries with daily alternating values.
In.flux = 5


# Create a solver
solver = cmf.CVodeIntegrator(p, 1e-8)

# Calculate results
Vsoil, Vsurf, = transpose([(c.layers[0].volume, c.surfacewater.volume)
                             for t in solver.run(cmf.Time(1,1,2012),cmf.Time(2,1,2012),cmf.min)])

# Present results
plt.figure()
plt.plot(Vsurf,label='Surface')
plt.plot(Vsoil,label='Soil')
plt.ylabel('Water content in mm')
plt.legend(loc=0)
plt.show()


import geometry as geo

obj = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\livestock\test\test_data\obj_to_shp\mesh.obj'
shp = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\livestock\test\test_data\obj_to_shp\shape_mesh.shp'

geo.obj_to_shp(obj, shp)



__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# -------------------------------------------------------------------------------------------------------------------- #
# Imports

# Module imports
import cmf
import pytest
import os

# Livestock imports
from livestock import hydrology


# -------------------------------------------------------------------------------------------------------------------- #
# Livestock Test


def test_load_cmf_files(input_files):
    (ground, mesh_paths, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = hydrology.load_cmf_files(input_files)

    assert ground
    assert isinstance(ground, list)
    for ground_ in ground:
        assert isinstance(ground_, dict)

    assert mesh_paths
    assert isinstance(mesh_paths, list)
    for mesh in mesh_paths:
        assert isinstance(mesh, str)
        assert os.path.split(mesh)[1].startswith('mesh')
        assert os.path.split(mesh)[1].endswith('.obj')

    assert outputs
    assert isinstance(outputs, dict)

    assert solver_settings
    assert isinstance(solver_settings, dict)


def test_mesh_to_cells(obj_file_paths):
    project = cmf.project()

    hydrology.mesh_to_cells(project, [obj_file_paths, ], False)

    assert project
    assert project.cells


def test_create_retention_curve(cmf_data):
    (ground_list, mesh_path, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data

    for ground in ground_list:
        curve_dict = ground['ground_type']['retention_curve']
        r_curve = hydrology.create_retention_curve(curve_dict)

        assert r_curve
        assert isinstance(r_curve, cmf.VanGenuchtenMualem)
        assert r_curve.Ksat == curve_dict['k_sat']
        assert r_curve.alpha == curve_dict['alpha']
        assert r_curve.Phi == curve_dict['phi']
        assert r_curve.n == curve_dict['n']
        assert r_curve.m == curve_dict['m']
        assert r_curve.l == curve_dict['l']


@pytest.mark.skip('Something wrong with the setup')
def test_install_cell_connections(cmf_data, project_with_cells):
    (ground_list, mesh_path, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data

    for ground in ground_list:
        for cell_index in ground['mesh']:
            cell = project_with_cells.cells[cell_index]
            hydrology.install_cell_connections(cell, ground['et_method'])


def test_build_cell(cmf_data, project_with_cells, retention_curve):
    (ground_list, mesh_paths, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data

    project, mesh_info = project_with_cells
    for ground in ground_list:
        for cell_index in mesh_info[ground['mesh']]:
            hydrology.build_cell(cell_index, project,
                                 ground, retention_curve)

            cell = project.cells[cell_index]

            assert cell
            if len(cell.layers) > 1:
                assert len(cell.layers) == len(ground['mesh']['layers'])
            assert cell.evaporation
            assert cell.transpiration
            assert cell.vegetation
            assert pytest.approx(cell.saturated_depth == ground['ground_type'][
                'saturated_depth'])

    assert True


def test_install_flux_connections(cmf_data, project_with_cells):
    (ground_list, mesh_path, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data
    project, mesh_info = project_with_cells

    for ground in ground_list:
        hydrology.install_flux_connections(project, ground)

    # TODO: Create better assessments
    assert project


def test_configure_cells(cmf_data, project_with_cells):
    (ground_list, mesh_path, weather_dict, trees_dict, outputs,
     solver_settings, boundary_dict) = cmf_data
    project, mesh_info = project_with_cells

    for ground in ground_list:
        hydrology.configure_cells(project, ground,  mesh_info[ground['mesh']])

    # TODO: Create better assessments
    assert project_with_cells


def test_add_tree_to_project():
    assert True


def test_create_weather():
    assert True


def test_create_boundary_conditions():
    assert True


def test_config_outputs():
    assert True


def test_gather_results():
    assert True


def test_get_analysis_length():
    assert True


def test_get_time_step():
    assert True


def test_solve_project(solve_ready_project, mock_solver, mock_gather_results):
    results = hydrology.solve_project(*solve_ready_project)

    assert results


def test_save_project():
    assert True


@pytest.mark.skip('Not ready yet')
def test_run_off(data_folder):
    folder = os.path.join(data_folder, 'run_off')
    hydrology.run_model(folder)



__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# -------------------------------------------------------------------------------------------------------------------- #
# Imports

# Module imports
import logging
import os
import shutil
import platform

# Livestock imports

# -------------------------------------------------------------------------------------------------------------------- #
# Livestock Loggers


def log_path():
    if not platform.system() == 'Linux':
        livestock_path = r'C:\livestock'
    else:
        livestock_path = os.path.dirname(__file__)

    log_folder = os.path.join(livestock_path, 'logs')

    if not os.path.exists(log_folder):
        os.mkdir(log_folder)

    return log_folder


def livestock_logger():

    if os.listdir(log_path()):
        shutil.rmtree(log_path())

    log = logging.getLogger(__name__)
    log.setLevel(logging.DEBUG)

    # Formatter
    formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(funcName)s'
                                  ' - %(message)s')

    # StreamHandler
    stream = logging.StreamHandler()
    stream.setLevel(logging.INFO)
    stream_formatter = logging.Formatter('%(message)s')
    stream.setFormatter(stream_formatter)

    # FileHandlers
    file_info = logging.FileHandler(os.path.join(log_path(),
                                                 'livestock_info.log'))
    file_info.setLevel(logging.INFO)
    file_info.setFormatter(formatter)

    file_debug = logging.FileHandler(os.path.join(log_path(),
                                                  'livestock_debug.log'))
    file_debug.setLevel(logging.DEBUG)
    file_debug.setFormatter(formatter)

    log.addHandler(stream)
    log.addHandler(file_debug)
    log.addHandler(file_info)

    return log



import hydrology as hy
import cmf
import shutil

def unpack(folder):
    test_files = folder + '/test_files.zip'
    shutil.unpack_archive(test_files, folder)

folder_path = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\livestock\test\test_data\cmf_slope'
unpack(folder_path)

model = hy.CMFModel(folder_path)
case = model.run_model()

cmf.describe(case, out=open(folder_path + '/case_description.txt','w'))



import cmf
from cmf.geos_shapereader import Shapefile as cmf_shape

p=cmf.project()
r_curve=cmf.VanGenuchtenMualem()

# Load cell polygons from file
path = r'C:\Users\Christian\Desktop\Livestock\shapefiles\shape_mesh.shp'
polygons=cmf_shape(path)

# create cell mesh from polygons
cells = p.cells_from_polygons(polygons)

for c in p.cells:
    # Add ten layers
    for i in range(10):
        c.AddLayer((i+1)*0.1,r_curve)
    cmf.Richards.use_for_cell(c)
    c.surfacewater_as_storage()
# subsurface flow
cmf.connect_cells_with_flux(p,cmf.Richards_lateral)
# surface runoff
cmf.connect_cells_with_flux(p,cmf.Manning_Kinematic)


__author__ = "Christian Kongsgaard"
__license__ = "GNU GPLv3"

# -------------------------------------------------------------------------------------------------------------------- #
# Imports

# Module imports
import cmf
from cmf.geos_shapereader import Shapefile
import cmf.geometry
import datetime
import numpy as np
import xml.etree.ElementTree as ET
import os
import xmltodict
import ast
import json
import typing
import progressbar

# Livestock imports
from livestock import geometry as lg
from livestock.log import livestock_logger

logger = livestock_logger()


# -------------------------------------------------------------------------------------------------------------------- #
# CMF Functions and Classes


def mesh_to_cells(cmf_project: cmf.project, mesh_paths: typing.List[str],
                  delete_after_load=False) -> typing.Tuple[cmf.project, dict]:
    """
    Takes a mesh and converts it into CMF cells

    :param mesh_paths: Path to mesh .obj file
    :param cmf_project: CMF project object.
    :param delete_after_load: If True, it deletes the input files after they have been loaded.
    :return: True
    """

    cell_dict = {}
    cell_counter = 0
    for mesh_file in mesh_paths:
        mesh_name = os.path.split(mesh_file)[1][:-4]
        cell_dict[mesh_name] = list()
        # Convert obj to shapefile
        shape_path = os.path.split(mesh_file)[0] + '/mesh.shp'
        lg.obj_to_shp(mesh_file, shape_path)
        polygons = Shapefile(shape_path)
        logger.debug('Converted .obj to .shp')

        for polygon in polygons:
            cmf.geometry.create_cell(cmf_project, polygon.shape,
                                     polygon.height, polygon.id)
            cell_dict[mesh_name].append(cell_counter)
            cell_counter += 1

        # Build topology
        cmf.geometry.mesh_project(cmf_project, verbose=False)
        logger.info('Build cells in project')

        if delete_after_load:
            os.remove(mesh_file)
            os.remove(shape_path)
            os.remove(os.path.split(mesh_file)[0] + '/mesh.dbf')
            os.remove(os.path.split(mesh_file)[0] + '/mesh.shx')
            logger.debug('Removed mesh files')

    return cmf_project, cell_dict


def set_vegetation_properties(cell_: cmf.Cell, property_dict: dict):
    """
    Sets the vegetation properties for a cell.

    :param cell_: Cell to set properties for.
    :type cell_: cmf.Cell
    :param property_dict: Dict containing the needed properties.
    :type property_dict: dict
    :return: True
    :rtype: bool
    """

    cell_.vegetation.Height = float(property_dict['height'])
    cell_.vegetation.LAI = float(property_dict['lai'])
    cell_.vegetation.albedo = float(property_dict['albedo'])
    cell_.vegetation.CanopyClosure = float(property_dict['canopy_closure'])
    cell_.vegetation.CanopyParExtinction = float(property_dict['canopy_par'])
    cell_.vegetation.CanopyCapacityPerLAI = float(property_dict['canopy_capacity'])
    cell_.vegetation.StomatalResistance = float(property_dict['stomatal_res'])
    cell_.vegetation.RootDepth = float(property_dict['root_depth'])
    cell_.vegetation.fraction_at_rootdepth = float(property_dict['root_fraction'])
    cell_.vegetation.LeafWidth = float(property_dict['leaf_width'])

    logger.debug(f'Sat vegetation properties for cell at: {cell_.get_position()}')

    return cell_


def add_tree_to_project(cmf_project, cell_index, property_dict):
    """
    Adds a tree to the model and sets the need properties for it.

    :param cmf_project: CMF project
    :type cmf_project: cmf.project
    :param cell_index: Index of the cell where the tree should be added.
    :type cell_index: int
    :param property_dict: Dict with tree properties.
    :type property_dict: dict
    :return: True
    :rtype: bool
    """

    cell = cmf_project.cells[int(cell_index)]
    set_vegetation_properties(cell, property_dict)
    name = 'canopy_' + str(cell_index)
    cell.add_storage(name, 'C')

    cmf.Rainfall(cell.canopy, cell, False, True)
    cmf.Rainfall(cell.surfacewater, cell, True, False)
    cmf.RutterInterception(cell.canopy, cell.surfacewater, cell)
    cmf.CanopyStorageEvaporation(cell.canopy, cell.evaporation, cell)

    logger.debug(f'Added a tree to cell at: {cell.get_position()}')

    return cmf_project


def load_cmf_files(folder: str, delete_after_load=False) -> tuple:
    """
    Loads the needed files for the CMF project to run.

    :param folder: Folder where the input files are located
    :param delete_after_load: Delete after the files are loaded?
    :return: True
    """

    # Load files and assign data to variables
    ground = load_ground(folder, delete_after_load)
    mesh_paths = load_meshes(folder)

    weather_dict = load_input_data(os.path.join(folder, 'weather.json'),
                                   delete_after_load)
    trees_dict = load_input_data(os.path.join(folder, 'tree.json'),
                                 delete_after_load)
    outputs = load_input_data(os.path.join(folder, 'outputs.json'),
                              delete_after_load)
    solver_settings = load_input_data(os.path.join(folder, 'solver.json'),
                                      delete_after_load)
    boundary_dict = load_input_data(os.path.join(folder,
                                                 'boundary_condition.json'),
                                    delete_after_load)

    logger.info('Loaded input files')

    return (ground, mesh_paths, weather_dict, trees_dict, outputs,
            solver_settings, boundary_dict)


def load_input_data(path: str, delete: bool) -> typing.Optional[dict]:
    # look for file
    if os.path.isfile(path):

        with open(path, 'r') as file:
            input_dict = json.load(file)

        # delete file
        if delete:
            os.remove(path)

        return input_dict

    else:
        logger.info(f'{os.path.split(path)[1]} not found')
        return None


def load_ground(folder: str, delete: bool) -> list:
    # look for file
    if os.path.isfile(folder + '/ground.json'):
        ground_path = folder + '/ground.json'

        with open(ground_path, 'r') as file:
            ground = json.load(file)

        # delete file
        if delete:
            os.remove(ground_path)

        return ground

    else:
        logger.error(f'Cannot find ground.json in folder: {folder}')
        raise FileNotFoundError(f'Cannot find ground.json in folder: '
                                f'{folder}')


def load_meshes(folder: str) -> typing.List[str]:
    # look for file
    meshes = []
    for file in os.listdir(folder):
        if file.endswith('.obj'):
            meshes.append(os.path.join(folder, file))

    if meshes:
        return meshes
    else:
        error = f'Cannot find any .obj files in folder: {folder}'
        logger.error(error)
        raise FileNotFoundError(error)


def create_retention_curve(retention_curve: dict) -> cmf.VanGenuchtenMualem:
    """
    Converts a dict of retention curve parameters into a CMF van
    Genuchten-Mualem retention curve.

    :param retention_curve: dict
    :return: CMF retention curve
    """

    curve = cmf.VanGenuchtenMualem(retention_curve['k_sat'],
                                   retention_curve['phi'],
                                   retention_curve['alpha'],
                                   retention_curve['n'],
                                   retention_curve['m'])
    curve.l = retention_curve['l']

    logger.debug(f'Created retention curve.')

    return curve


def install_cell_connections(cell: cmf.Cell, evapotranspiration_method: str) \
        -> cmf.Cell:
    # Install connections
    cell.install_connection(cmf.Richards)
    cell.install_connection(cmf.SimpleInfiltration)

    if evapotranspiration_method == 'penman_monteith':
        # Install Penman & Monteith method to calculate evapotranspiration
        # potential
        cell.install_connection(cmf.PenmanMonteithET)

        # Install surface water evaporation
        cmf.PenmanEvaporation(cell.surfacewater, cell.evaporation,
                              cell.meteorology)

        logger.debug(f'Install Richards connection, simple infiltration, '
                     f'Penman-Monteith evapotranspiration and '
                     f'Penman evaporation for cell at: {cell.get_position()}')

    elif evapotranspiration_method == 'shuttleworth_wallace':
        # Install Shuttleworth-Wallace method to calculate evapotranspiration
        cell.install_connection(cmf.ShuttleworthWallace)

        # Install surface water evaporation
        cmf.PenmanEvaporation(cell.surfacewater, cell.evaporation,
                              cell.meteorology)

        logger.debug(f'Install Richards connection, simple infiltration, '
                     f'Shuttleworth-Wallace evapotranspiration and '
                     f'Penman evaporation for cell at: {cell.get_position()}')

    return cell


def install_flux_connections(cmf_project: cmf.project, cell_properties: dict) -> cmf.project:
    cmf.connect_cells_with_flux(cmf_project, cmf.DarcyKinematic)
    logger.info('Installed Darcy kinematic for all cells in project.')

    if cell_properties['runoff_method'] == 'kinematic':
        cmf.connect_cells_with_flux(cmf_project, cmf.KinematicSurfaceRunoff)
        logger.info('Installed kinematic surface run-off')

    elif cell_properties['runoff_method'] == 'diffusive':
        cmf.DiffusiveSurfaceRunoff.set_linear_slope(1e-8)
        cmf.connect_cells_with_flux(cmf_project, cmf.DiffusiveSurfaceRunoff)
        logger.info('Installed diffusive surface run-off. Slope set to 1e-8.')

    return cmf_project


def build_cell(cell_id: int, cmf_project: cmf.project, cell_properties: dict,
               r_curve: cmf.VanGenuchtenMualem) -> cmf.project:
    cell = cmf_project.cells[cell_id]
    logger.debug(f'Building cell with ID: {cell_id}')

    # Add layers
    for i in range(len(cell_properties['layers'])):
        cell.add_layer(float(cell_properties['layers'][i]), r_curve)

    install_cell_connections(cell, cell_properties['et_method'])

    set_vegetation_properties(cell, cell_properties['ground_type']['surface_properties'])

    if cell_properties['ground_type']['manning']:
        cell.surfacewater.set_nManning(cell_properties['ground_type']['manning'])
        logger.debug(f'Sat Manning roughness to: {cell_properties["ground_type"]["manning"]}')

    if cell_properties['ground_type']['puddle_depth']:
        cell.surfacewater.puddledepth = cell_properties['ground_type']['puddle_depth']
        logger.debug(f'Sat puddle depth to: {cell_properties["ground_type"]["puddle_depth"]}')

    if cell_properties['surface_water']:
        cell.surfacewater.volume = cell_properties['surface_water']
        logger.info(f'Sat surface water to {cell_properties["surface_water"]} for cell with ID: {cell_id}')

    # Set initial saturation
    cell.saturated_depth = cell_properties['ground_type']['saturated_depth']
    logger.debug(f'Sat saturated depth to {cell_properties["ground_type"]["saturated_depth"]}')

    return cmf_project


def configure_cells(cmf_project: cmf.project, cell_properties_dict: dict,
                    mesh_info: dict) -> cmf.project:
    """
    Configure and setup all needed information for the cells.

    :param cmf_project: CMF project
    :param cell_properties_dict: Dict with all the needed properties
    :return: cmf project
    """

    # Helper functions

    # Convert retention curve parameters into CMF retention curve
    r_curve = create_retention_curve(cell_properties_dict['ground_type']['retention_curve'])

    for cell_index in mesh_info:
        build_cell(cell_index, cmf_project, cell_properties_dict, r_curve)

    # Connect fluxes
    install_flux_connections(cmf_project, cell_properties_dict)

    logger.info('Configured all cells in project.')

    return cmf_project


def config_outputs(cmf_project: cmf.project, output_configuration: dict) -> dict:
    """
    Function to set up result gathering dictionary.

    :param cmf_project: CMF project to collect from
    :param output_configuration: Dict to configure the results from
    :return: Empty result dictionary
    """

    results = {}

    for cell_index in range(0, len(cmf_project.cells)):
        cell_name = 'cell_' + str(cell_index)
        results[cell_name] = {}

        # Set all cell related outputs
        for cell_output in output_configuration['cell']:
            results[cell_name][str(cell_output)] = []

        for layer_index in range(0, len(cmf_project.cells[cell_index].layers)):
            layer_name = 'layer_' + str(layer_index)
            results[cell_name][layer_name] = {}

            # Set all layer related outputs
            for layer_output in output_configuration['layer']:
                results[cell_name][layer_name][layer_output] = []

    logger.debug(f'Configured outputs for CMF project. Outputs was: {results}')

    return results


def gather_results(cmf_project: cmf.project, results: dict, time: datetime.datetime):
    """
    Collects the produced results.

    :param cmf_project: CMF project to collect from
    :param results: Dict to gather the results in
    :param time: Point in solver time
    :return:
    """

    logger.debug(f'Gathering results for time: {time}')

    for cell_index in range(0, len(cmf_project.cells)):
        cell_name = 'cell_' + str(cell_index)

        for out_key in results[cell_name].keys():

            # Collect cell related results
            if not out_key.startswith('layer'):
                if out_key == 'evaporation':
                    evap = cmf_project.cells[cell_index].evaporation

                    flux_at_time = 0
                    for flux, node in evap.fluxes(time):
                        flux_at_time += flux

                    results[cell_name][out_key].append(flux_at_time)

                elif out_key == 'transpiration':
                    transp = cmf_project.cells[cell_index].transpiration

                    flux_at_time = 0
                    for flux, node in transp.fluxes(time):
                        flux_at_time += flux

                    results[cell_name][out_key].append(flux_at_time)

                elif out_key == 'surface_water_volume':
                    volume = cmf_project.cells[cell_index].surfacewater.volume
                    results[cell_name][out_key].append(volume)

                elif out_key == 'surface_water_flux':
                    water = cmf_project.cells[cell_index].get_surfacewater()

                    flux_and_node = []
                    for flux, node in water.fluxes(time):
                        flux_and_node.append((flux, node))

                    results[cell_name][out_key].append(flux_and_node)

                elif out_key == 'heat_flux':
                    results[cell_name][out_key].append(cmf_project.cells[cell_index].heat_flux(time))

                elif out_key == 'aerodynamic_resistance':
                    results[cell_name][out_key].append(
                        cmf_project.cells[cell_index].get_aerodynamic_resistance(time))

        for layer_index in range(0, len(cmf_project.cells[cell_index].layers)):
            layer_name = 'layer_' + str(layer_index)

            for out_key in results[cell_name][layer_name].keys():

                # Collect layer related results

                if out_key == 'potential':
                    results[cell_name][layer_name][out_key].append(
                        cmf_project.cells[cell_index].layers[layer_index].potential)

                elif out_key == 'theta':
                    results[cell_name][layer_name][out_key].append(
                        cmf_project.cells[cell_index].layers[layer_index].theta)

                elif out_key == 'volumetric_flux':
                    layer = cmf_project.cells[cell_index].layers[layer_index].get_3d_flux(time)

                    """
                    flux_and_node = []
                    for flux, node in layer.fluxes(time):
                        flux_and_node.append((flux, node))
                    """

                    results[cell_name][layer_name][out_key].append(layer)

                elif out_key == 'volume':
                    results[cell_name][layer_name][out_key].append(
                        cmf_project.cells[cell_index].layers[layer_index].volume)

                elif out_key == 'wetness':
                    results[cell_name][layer_name][out_key].append(
                        cmf_project.cells[cell_index].layers[layer_index].wetness)


def get_analysis_length(analysis_length: list) -> datetime.timedelta:
    length, quantity = analysis_length

    if quantity == 'y':
        logger.debug(f'Analysis length set to: {length} {quantity}')
        return datetime.timedelta(days=length * 365)

    elif quantity == 'd':
        logger.debug(f'Analysis length set to: {length} {quantity}')
        return datetime.timedelta(days=length)

    elif quantity == 'h':
        logger.debug(f'Analysis length set to: {length} {quantity}')
        return datetime.timedelta(hours=length)

    elif quantity == 'm':
        logger.debug(f'Analysis length set to: {length} {quantity}')
        return datetime.timedelta(minutes=length)

    elif quantity == 's':
        logger.debug(f'Analysis length set to: {length} {quantity}')
        return datetime.timedelta(seconds=length)


def get_time_step(time_step: list) -> datetime.timedelta:
    step_size, quantity = time_step

    if quantity == 'y':
        logger.debug(f'Solver time step set to: {step_size} {quantity}')
        return datetime.timedelta(days=step_size * 365)

    elif quantity == 'd':
        logger.debug(f'Solver time step set to: {step_size} {quantity}')
        return datetime.timedelta(days=step_size)

    elif quantity == 'h':
        logger.debug(f'Solver time step set to: {step_size} {quantity}')
        return datetime.timedelta(hours=step_size)

    elif quantity == 'm':
        logger.debug(f'Solver time step set to: {step_size} {quantity}')
        return datetime.timedelta(minutes=step_size)

    elif quantity == 's':
        logger.debug(f'Solver time step set to: {step_size} {quantity}')
        return datetime.timedelta(seconds=step_size)


def solve_project(cmf_project: cmf.project, solver_settings: dict,
                  outputs: dict) -> dict:
    """Solves the model"""

    logger.info('Initializing solving of CMF project')

    # Create solver, set time and set up results
    solver = cmf.CVodeIntegrator(cmf_project,
                                 solver_settings['tolerance'])
    solver.t = cmf.Time(solver_settings['start_time']['day'],
                        solver_settings['start_time']['month'],
                        solver_settings['start_time']['year'])

    logger.debug(f'Solver start time: {solver.t}')

    results = config_outputs(cmf_project, outputs)

    # Save initial conditions to results
    gather_results(cmf_project, results, solver.t)

    analysis_length = get_analysis_length(solver_settings['analysis_length'])
    time_step = get_time_step(solver_settings['time_step'])
    number_of_steps = analysis_length.total_seconds() / time_step.total_seconds()

    # Run solver and save results at each time step
    widgets = [' [', progressbar.Timer(), '] ',
               progressbar.Bar(),
               ' [', progressbar.AdaptiveETA(), ' ]']
    bar = progressbar.ProgressBar(max_value=number_of_steps, widgets=widgets)
    for index, time_ in enumerate(solver.run(solver.t,
                                             solver.t + analysis_length,
                                             time_step)):
        gather_results(cmf_project, results, time_)
        bar.update(index)

    logger.info('Solved CMF project!')

    return results


def save_results(results: dict, folder: str):
    """Saves the computed results to a json file"""

    path = os.path.join(folder, 'results.json')

    logger.debug(f'Saving results to: {path}')

    with open(path, 'w') as file:
        json.dump(results, file)


def create_weather(cmf_project):
    raise NotImplementedError


def create_boundary_conditions(cmf_project):
    raise NotImplementedError


def run_model(folder: str) -> cmf.project:
    """
    Runs the model with everything.

    :return: Simulated CMF project
    """

    # Initialize project
    project = cmf.project()

    (ground_list, mesh_paths, weather_dict, trees_dict, outputs, solver_settings,
     boundary_dict) = load_cmf_files(folder, False)

    # Add cells and properties to them
    project, mesh_info = mesh_to_cells(project, mesh_paths)

    for ground in ground_list:
        configure_cells(project, ground, mesh_info[ground['mesh']])

    if trees_dict:
        for key in trees_dict.keys():
            add_tree_to_project(project,
                                trees_dict[key]['face_index'],
                                trees_dict[key]['property'])

    # Create the weather
    if weather_dict:
        create_weather(project)

    # Create boundary conditions
    if boundary_dict:
        create_boundary_conditions(project)

    # Run solver
    results = solve_project(project, solver_settings, outputs)

    # Save the results
    save_results(results, folder)

    return project


def cmf_results(path: str) -> bool:
    """
    Process a CMF result file.

    :param path: Folder containing the result file.
    :type path: str
    :return: True
    :rtype: bool
    """

    files = os.listdir(path)
    result_path = None
    lookup_path = None

    for f in files:
        if f.startswith('results'):
            result_path = path + '/' + f
        elif f.startswith('result_lookup'):
            lookup_path = path + '/' + f

    # Read look up file
    file_obj = open(lookup_path, 'r')
    line = file_obj.readline()
    lookup_dict = eval(line)

    for lookup_key in lookup_dict.keys():
        if lookup_key.startswith('cell'):
            cell_results(lookup_dict[lookup_key], result_path, path)
        elif lookup_key.startswith('layer'):
            layer_results(lookup_dict[lookup_key], result_path, path)

    return True


def cell_results(looking_for: str, result_file: str, folder: str) -> bool:
    """Processes cell results after a desired parameter.

    :param looking_for: Parameter to look for.
    :type looking_for: str
    :param result_file: Path of the result file.
    :type result_file: str
    :param folder: Path of the folder
    :type folder: str
    :return: True
    :rtype: bool
    """

    # Initialize
    result_tree = ET.tostring(ET.parse(result_file).getroot())
    results = xmltodict.parse(result_tree)
    results_to_save = []

    # Find results
    for cell in results['result'].keys():
        if looking_for == 'evapotranspiration':
            evapo = np.array(eval(results['result'][cell]['evaporation']))
            transp = np.array(eval(results['result'][cell]['transpiration']))
            evapotransp = evapo + transp
            results_to_save.append(list(evapotransp))

        else:
            for result in results['result'][cell]:
                if result == looking_for:
                    if result == 'heat_flux':
                        # Covert heat flux from MJ/(m2*day) to W/m2h
                        flux_mj = np.array(eval(results['result'][cell][result]))
                        flux_wh = flux_mj / 0.0864
                        results_to_save.append(list(flux_wh))

                    else:
                        results_to_save.append(eval(results['result'][cell][result]))

    # Write files
    file_path = folder + '/' + looking_for + '.csv'
    csv_file = open(file_path, 'w')
    for result_ in results_to_save:
        csv_file.write(','.join(str(r)
                                for r in result_) + '\n')
    csv_file.close()

    return True


def layer_results(looking_for: str, result_file: str, folder: str) -> bool:
    """
    Processes layer results after a desired parameter.

    :param looking_for: Parameter to look for.
    :type looking_for: str
    :param result_file: Path of the result file.
    :type result_file: str
    :param folder: Path of the folder
    :type folder: str
    :return: True
    :rtype: bool
    """

    # Initialize
    result_tree = ET.tostring(ET.parse(result_file).getroot())
    results = xmltodict.parse(result_tree)
    results_to_save = []

    # find results
    for cell in results['result'].keys():

        for cell_result in results['result'][cell].keys():
            if cell_result.startswith('layer'):
                for layer_result in results['result'][cell][cell_result]:
                    if layer_result == looking_for:
                        if layer_result == 'volumetric_flux':
                            results_to_save.append(convert_cmf_points(results['result']
                                                                      [cell]
                                                                      [cell_result]
                                                                      [layer_result]))

                        else:
                            results_to_save.append(eval(results['result'][cell][cell_result][layer_result]))

        results_to_save.append([cell])

    # Write files
    file_path = folder + '/' + looking_for + '.csv'
    csv_file = open(file_path, 'w')
    for result_ in results_to_save:
        csv_file.write(','.join(str(r) for r in result_) + '\n')
    csv_file.close()

    return True


def convert_cmf_points(points: str) -> list:
    """
    Convert a string of CMF points into a list of tuples with their coordinates.

    :param points: CMF points
    :type points: str
    :return: List of tuples contain point coordniates
    :rtype: list
    """

    # convert to list
    point_list = points[1:-1]
    point_tuples = []

    for tup in point_list.split(' '):
        try:
            tup1 = ast.literal_eval(tup[9:-1])
        except SyntaxError:
            tup1 = ast.literal_eval(tup[9:])
        point_tuples.append(' '.join(str(e) for e in tup1))

    return point_tuples


def surface_flux_results(path: str) -> bool:
    """
    Generates the surface flux from a CMF result file.

    :param path: Folder where the result file is located.
    :type path: str
    :return: True
    :rtype: bool
    """

    # Helper functions
    def read_files(path_):

        # get flux configuration file
        if os.path.isfile(path_ + '/flux_config.txt'):
            file_obj = open(path_ + '/flux_config.txt', 'r')
            flux_lines = file_obj.readlines()
            file_obj.close()
            os.remove(path_ + '/flux_config.txt')

            flux = {'run_off': ast.literal_eval(flux_lines[0].strip()),
                    'rain': ast.literal_eval(flux_lines[1].strip()),
                    'evapotranspiration': ast.literal_eval(flux_lines[2].strip()),
                    'infiltration': ast.literal_eval(flux_lines[3].strip())
                    }
        else:
            raise FileNotFoundError('Cannot find flux_config.txt in folder: ' + str(path_))

        # get center points file
        if os.path.isfile(path_ + '/center_points.txt'):
            file_obj = open(path_ + '/center_points.txt', 'r')
            point_lines = file_obj.readlines()
            file_obj.close()
            os.remove(path_ + '/center_points.txt')

            # process points
            center_points_ = []
            for line_ in point_lines:
                point_ = []
                for p in line_.strip().split(','):
                    point_.append(float(p))
                center_points_.append(np.array(point_))

        else:
            raise FileNotFoundError('Cannot find center_points.txt in folder: ' + str(path_))

        return flux, center_points_

    def get_flux_result(path_: str) -> dict:

        # helper
        def convert_to_list(string_: str) -> list:

            time_steps = []
            current_time = []
            flux_pair = []
            for element in string_[1:-1].split(','):
                if element.startswith('['):
                    flux_pair.append(float(element[2:]))

                elif element.startswith(' ['):
                    flux_pair.append(float(element[3:]))

                elif element.startswith(' ('):
                    flux_pair.append(float(element[2:]))

                elif element.endswith(')'):
                    flux_pair.append(element[1:-1])
                    current_time.append(flux_pair)
                    flux_pair = []

                elif element.endswith(']'):
                    flux_pair.append(element[1:-2])
                    current_time.append(flux_pair)
                    time_steps.append(current_time)
                    current_time = []
                    flux_pair = []

            return time_steps

        result_file = path_ + '/results.xml'

        # make check for file
        if not os.path.isfile(result_file):
            raise FileNotFoundError('Cannot find results.xml in folder: ' + str(path_))

        # Initialize
        result_tree = ET.tostring(ET.parse(result_file).getroot())
        results = xmltodict.parse(result_tree)
        results_to_save = {}

        # find results
        for cell_ in results['result'].keys():
            results_to_save[cell_] = convert_to_list(str(results['result'][cell_]['surface_water_flux']))

        return results_to_save

    def process_flux(flux_dict_: dict, flux_config_: dict, center_points_: list) -> dict:

        # helper functions
        def get_wanted_fluxes(flux_tuple: list, center_points__) -> np.array:
            # delete entries where the flux is 0
            if flux_tuple[0] == 0:
                return np.array([])

            else:
                # delete entries that we don't want, if flux is not zero
                if flux_tuple[1].startswith('{Rain'):
                    if not flux_config_['rain']:
                        return np.array([])
                    else:
                        return np.array([0, 0, flux_tuple[0]])

                elif flux_tuple[1].startswith('{Evapo'):
                    if not flux_config_['evapotranspiration']:
                        return np.array([])
                    else:
                        return np.array([0, 0, flux_tuple[0]])

                elif flux_tuple[1].startswith('{Layer'):
                    if not flux_config_['infiltration']:
                        return np.array([])
                    else:
                        return np.array([0, 0, -flux_tuple[0]])

                elif flux_tuple[1].startswith('{Surface'):
                    if not flux_config_['run_off']:
                        return np.array([])
                    else:
                        if flux_tuple[0] > 0:
                            return np.array([])
                        else:
                            destination_cell = int(flux_tuple[1].split('#')[1][:-1])
                            local_cell = int(cell_.split('_')[1])
                            vector_ = center_points__[destination_cell] - center_points__[local_cell]
                            normalized_vector = vector_ / np.linalg.norm(vector_)
                            return normalized_vector * abs(flux_tuple[0])

        cell_vectors = {}
        for cell_ in flux_dict_.keys():
            time_vectors = []
            for time_step in range(0, len(flux_dict_[cell_])):
                vectors = []

                for flux in flux_dict_[cell_][time_step]:
                    vector = get_wanted_fluxes(flux, center_points_)
                    if len(vector) > 0:
                        vectors.append(vector)

                # Compute average vector
                if not vectors:
                    # if no vectors return [0, 0, 0]
                    time_vectors.append([0, 0, 0])
                else:
                    time_vectors.append(list(np.average(vectors, axis=0)))

            # add time vectors to cell
            cell_vectors[cell_] = time_vectors

        return cell_vectors

    # load files
    flux_config, center_points = read_files(path)
    flux_results = get_flux_result(path)

    processed_vectors = process_flux(flux_results, flux_config, center_points)

    # write file
    result_obj = open(path + '/surface_flux_result.txt', 'w')
    for cell in processed_vectors.keys():
        line = ''
        for point in processed_vectors[cell]:
            line += ','.join([str(p) for p in point]) + '\t'
        result_obj.write(line + '\n')

    result_obj.close()

    return True



__author__ = "Christian Kongsgaard"
__license__ = "MIT"

# ---------------------------------------------------------------------------- #
# Imports

from setuptools import setup
from codecs import open
from os import path
import datetime

# ---------------------------------------------------------------------------- #
here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

now = datetime.datetime.now()
release_version = '2'
version = f'{now.year}.{now.month}.{release_version}'

setup(
    name='livestock',
    version=version,
    description='Livestock is a plugin/library for Grasshopper '
                'written in Python',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/ocni-dtu/livestock',
    author='Christian Kongsgaard',
    author_email='ocni@dtu.dk',
    license='GNU GPLv3',
    keywords='hydrology 3dmodeling grasshopper',
    packages=['livestock', ],
    install_requires=['scipy', 'numpy', 'paramiko', 'pyshp', 'shapely',
                      'cmf==1.2', 'xmltodict', 'dbfread', 'progressbar2'],
    python_requires='>3.x',
    #entry_points={'console_scripts': ['livestock=livestock:main', ], },
)



import numpy as np
from functools import wraps
import os


def file_as_input(function_):

    @wraps(function_)
    def wrapper(*args, **kwargs):

        args_ = []
        for arg in args:
            try:
                os.path.isfile(arg)
                args_.append(np.loadtxt(arg))
            except ValueError:
                args_.append(arg)

        return function_(*args_, **kwargs)
    return wrapper


def poly_list(x):
    return x**2 + 5*x - 2


@file_as_input
def poly_file(x):
    return x**2 + 5*x - 2




data_file = r'C:\Users\Christian\Dropbox\Arbejde\DTU BYG\Livestock\livestock\livestock\test\local_test\data.txt'
data_list = np.array([1, 2, 3, 4, 5, 6, 7, 8, 9, 10])


print(poly_file(data_list))

print(poly_file(data_file))
